/* fir-tools/src/fir3db/800_fir3db.c
 *
 *  Copyright: ©2014, Laurence Withers
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
 */



enum {
    /* number of points for initial coarse search */
    npoints = 1000,

    /* number of iterations when finding -3dB point */
    niter = 10,
};



void
fir3db_aux(struct fir_filter_t* fi, double freq1, double freq2, double gain)
{
    int i;
    double complex resp;
    double fmid, mag;

    for(i = 0; i < niter; ++i) {
        fmid = (freq1 + freq2) / 2;
        resp = fir_filter_response(fi, fmid);
        mag = cabs(resp);

        if(mag > gain) {
            freq2 = fmid;
        } else {
            freq1 = fmid;
        }
    }

    printf("%.8g\n", fmid);
}



void
fir3db(struct fir_filter_t* fi, double gain)
{
    double freq, freq_last, mag, mag_last;
    double complex resp;
    int i;

    resp = fir_filter_response(fi, 0);
    mag_last = cabs(resp);
    freq_last = 0;

    for(i = 1; i < npoints; ++i) {
        freq = (i * 0.5) / (npoints - 1);
        resp = fir_filter_response(fi, freq);
        mag = cabs(resp);

        if(mag_last < gain && mag >= gain) {
            fir3db_aux(fi, freq_last, freq, gain);
        }

        if(mag_last >= gain && mag < gain) {
            fir3db_aux(fi, freq, freq_last, gain);
        }

        mag_last = mag;
        freq_last = freq;
    }
}



/* options for text editors
vim: expandtab:ts=4:sw=4
*/
